/*
  This code shows how to write a SDL Text Class using a normal SDL window,
  it uses SDL_ttf as a text creator.
  It also uses the Unicode keyboard support from SDL.

  For any problem of installation or usage, please send me a message : 
	http://www.developpez.net/forums/private.php?do=newpm&u=78956

  Copyright (C) 2007 Beyler Jean Christophe

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <SDL.h>
#include <SDL_ttf.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "CTexte.h"

/* Fonctions statiques */
static void Texte_CreerTexte();

/* Constructeur */
STexte *Texte_Creer()
{
    STexte *res = malloc(sizeof(*res));

    if(res == NULL) {
        return NULL;
    }
    
    /* Valeurs par defaut */
    res->surface = NULL;
    res->message = NULL;
    res->message_len = 0;
    res->font = NULL;
    res->font_a_nous = 0;
    res->rect.x = 0;
    res->rect.y = 0;

    return res;
}

/* Destructeur */
void Texte_Detruire(STexte **txt)
{
    if(txt != NULL) {
        if(*txt != NULL) {
            /* Si la police est a nous, on la libere */
            if((*txt)->font_a_nous) {
                TTF_CloseFont((*txt)->font);
                SDL_FreeSurface((*txt)->surface);
            }
            /* Liberation */
            free(*txt);
            *txt = NULL;
        }
    }
}

/* Chargement d'une police, elle ne sera pas a nous, donc le destructeur ne la liberera pas */
int Texte_MettrePolice(STexte *txt, TTF_Font *font, int anous)
{
    if(txt != NULL) {
        txt->font = font;
        txt->font_a_nous = anous;
        return 1;
    }
    return 0;
}

/* Chargement d'une police */
int Texte_ChargerPolice(STexte* txt, char * police, int size, int R, int G, int B)
{
    if(txt != NULL) {
        txt->color.r=R;
        txt->color.g=G;
        txt->color.b=B;

        txt->font = TTF_OpenFont(police, size);
        if(txt->font==NULL) {
            return 0;
        }
        return 1;
    }
    
    return 0;
}

/* Dessiner le texte, gardant le message et la position */
int Texte_AffTexte(STexte *txt, SDL_Surface *ecran)
{
    if(txt != NULL) {
        if(txt->surface != NULL) {
            SDL_BlitSurface(txt->surface, NULL, ecran, &(txt->rect));
            return 1;
        }
    }
    return 0;
}

/* Dessiner le texte, gardant le texte */
int Texte_AffTexte_Position(STexte *txt, SDL_Surface *ecran, int x, int y)
{
    if(txt != NULL) {
        txt->rect.x = x;
        txt->rect.y = y;
        return Texte_AffTexte(txt, ecran);
    }
    return 0;
}

/* Creer le texte */
void Texte_CreerTexte(STexte *txt)
{
    if(txt != NULL) {
        if(txt->font!=NULL) {
            if(txt->surface != NULL) {  
                SDL_FreeSurface(txt->surface);  
                txt->surface = NULL;       
            }
            if(txt->message != NULL) {
                /* Creation de la texture du texte */
                SDL_Surface *temp;

                temp = TTF_RenderText_Blended(txt->font, txt->message, txt->color);

                if(temp != NULL) {
                    /* Meme format */
                    SDL_Surface *temp2 = SDL_DisplayFormatAlpha(temp);

                    /* Si cela ne fonctionne pas, on garde temp... */
                    if(temp2 != NULL) {
                        /* Liberation */
                        SDL_FreeSurface(temp);
                        temp = temp2;
                    }

                    /* Affectation de la surface */
                    txt->surface = temp;
                }
            }
            else {
                txt->surface = NULL;
            }
        }
    }
}

/* Dessiner le texte, gardant la position */
int Texte_AffTexte_Texte(STexte *txt, SDL_Surface *ecran, char *message)
{
    size_t len;

    if(txt!=NULL) {
        if(txt->font != NULL) {
            if( (txt->message == NULL) || (strcmp(message, txt->message) != 0)) {
                if(txt->message != NULL) {
                    SDL_FreeSurface(txt->surface);
                    txt->surface = NULL;
                }

                len = strlen(message) + 1; /* +1 pour le '\0' */

                /* On teste la longueur */
                if(len > txt->message_len) {
                    /* On doit allouer plus */
                    free(txt->message);
                    txt->message = malloc(len);

                    if(txt->message == NULL) {
                        return 0;
                    }

                    /* On copie */
                    strcpy(txt->message, message);
                    txt->message_len = len;
                }

                /* Creer texte */
                Texte_CreerTexte(txt);

            }
        }
        return Texte_AffTexte(txt, ecran);
    }
    return 0;
}

int Texte_AffTexte_PositionTexte(STexte *txt, SDL_Surface *ecran, char *message, int x, int y)
{
    if(txt != NULL) {
        txt->rect.x = x;
        txt->rect.y = y;
        return Texte_AffTexte_Texte(txt, ecran, message);
    }
    return 0;
}

void Texte_SetPosition(STexte *txt, int x, int y)
{
    if(txt != NULL) {
        txt->rect.x = x;
        txt->rect.y = y;
    }
}

void Texte_AjouteFin(STexte *txt, char c)
{
    if(txt != NULL) {
        if( c != '\0') {
            size_t len = strlen(txt->message);

            /* A-t-on encore de la place */
            if(len >= txt->message_len) {
                size_t nouvelletaille = txt->message_len * 2 + 1;
                char *tmp = realloc(txt->message, nouvelletaille);

                /* Gestion d'erreur -> on sort */
                if(tmp == NULL) {
                    return;
                }

                txt->message = tmp;
                txt->message_len = nouvelletaille;
            }

            txt->message[len] = c;
            len++;
            txt->message[len] = '\0';
            
            Texte_CreerTexte(txt);
        }
    }
}

void Texte_EnleveFin(STexte *txt)
{
    if(txt != NULL) {
        if(txt->message != NULL) {
            size_t len = strlen(txt->message);

            if(len > 0) {
                len--;
                txt->message[len] = '\0';
                Texte_CreerTexte(txt);
            }
        }
    }
}
